/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import java.io.File;
import java.io.IOException;

import javax.persistence.EntityManager;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import ch.hortis.sonar.model.Snapshot;
import ch.hortis.sonar.service.SnapshotService;
import ch.hortis.sonar.service.WebInterfaceService;
import ch.hortis.sonar.service.WebInterfaceServiceImpl;

/**
 * Creates the snapshots hierarchy and launch the batch on the sonar server side
 * @goal batch
 * @aggregator
 */
public class BatchLaunchMojo extends CoreMojo {
  
  /**
   * Sonar host URL
   * @parameter expression="${sonar.host.url}" default-value="http://localhost:9000" alias="sonar.host.url"
   */
  private String sonarHostURL;
  
  private WebInterfaceService webInterfaceService;
  
  public void setWebInterfaceService(WebInterfaceService webInterfaceService) {
    this.webInterfaceService = webInterfaceService;
  }

  public void execute() throws MojoExecutionException {
    if (webInterfaceService == null) {
      webInterfaceService = new WebInterfaceServiceImpl(sonarHostURL);
    }
    
    EntityManager manager = getEntityManager();
    try {
      manager.getTransaction().begin();
      processSnapshots();
      manager.getTransaction().commit();
      
      Snapshot topLevel = getTopLevelSnapshot();
      if ( topLevel == null ) {
        throw new MojoExecutionException("Unable to find top level snapshot");
      }
      try {
        getLog().debug("Firing root snapshot " + topLevel.getId());
        webInterfaceService.triggerMeasuresCalculations(topLevel.getId());
      } catch (IOException e) {
        throw new MojoExecutionException("Error occured when triggering measures calculation job on remote server", e);
      }
    } finally {
      if (manager.getTransaction().isActive()) {
        manager.getTransaction().rollback();
      }
      webInterfaceService = null;
      destroyManager();
    }
  }
  
  private Snapshot getTopLevelSnapshot() throws MojoExecutionException {
    SnapshotService service = new SnapshotService(getEntityManager());
    for (org.apache.maven.project.MavenProject reactorProject : getReactorProjects()) {
      Integer projectBoundSnapshotId = super.getSnapshotId(reactorProject);
      Snapshot projectBoundSnapshot = getEntityManager().find(Snapshot.class, projectBoundSnapshotId);
      if (projectBoundSnapshot.getParent() == null) {
        if (getReactorProjects().size() == 1 || 
            service.getChildSnapshots(projectBoundSnapshot, false).size() > 0) {
          return projectBoundSnapshot;
        }
      }
    }
    return null;
  }
  
  private void processSnapshots() throws MojoExecutionException {
    for (MavenProject reactorProject : getReactorProjects()) {
      processSnapshot(reactorProject, null);
    }
  }
  
  private void processSnapshot(MavenProject reactorProject, Snapshot parent) throws MojoExecutionException {
    Integer projectBoundSnapshotId = super.getSnapshotId(reactorProject);
    Snapshot projectBoundSnapshot = getEntityManager().find(Snapshot.class, projectBoundSnapshotId);
    if ( projectBoundSnapshot == null ) {
      throw new MojoExecutionException("Unable to find Snapshot " + projectBoundSnapshotId + " bound to the pom");
    }
    if (parent != null) {
      projectBoundSnapshot.setParent(parent);
      if ( getLog().isDebugEnabled() ) {
        getLog().debug(getSonarProjectId(projectBoundSnapshot.getMavenProject()) + "(" + projectBoundSnapshot.getId() + ") ->" + 
                       getSonarProjectId(parent.getMavenProject()) + " (" + parent.getId() + ")");
      }
    }

    projectBoundSnapshot.setStatus(Snapshot.STATUS_TO_PROCESS);
    getEntityManager().merge(projectBoundSnapshot);
    
    saveSnapshotHierarchyModules(projectBoundSnapshot, reactorProject);
  }

  private void saveSnapshotHierarchyModules(Snapshot snapshot, MavenProject reactorProject) throws MojoExecutionException {
    for (Object o : reactorProject.getModules()) {
      String moduleName = (String) o;
      File moduleFile = new File(reactorProject.getBasedir() + "/" + moduleName + "/pom.xml");
      // cannot use the readen module Project to create the hierarchy, must use one of
      // the maven object in the getReactorProjects() array
      MavenProject targetReactorProject = null;
      for( MavenProject project : getReactorProjects() ) {
        try {
          if (project.getFile().getCanonicalFile().equals(moduleFile.getCanonicalFile())) {
            targetReactorProject = project;
            break;
          }
        } catch (IOException e) {
          throw new MojoExecutionException("Unable to resolve canonical path of pom.xml", e);
        }
      }
      if ( targetReactorProject == null ) {
        throw new MojoExecutionException("Unable to find module " + moduleName + " in reactor projects");
      }
      processSnapshot(targetReactorProject, snapshot);
    }
  }
  
  private String getSonarProjectId(ch.hortis.sonar.model.MavenProject project) {
    return project.getGroupId() + ":" + project.getArtifactId() + ":" + project.getBranch();
  }
  
}