/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import java.sql.Connection;
import java.util.List;

import javax.persistence.EntityManager;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import ch.hortis.sonar.jpa.Persistence;
import ch.hortis.sonar.jpa.PersistenceException;
import ch.hortis.sonar.model.JdbcData;

public abstract class CoreMojo extends AbstractMojo {
  
  private final static String SNAPSHOT_ID_PROP = "sonar.snaphsotId";
  
  /**
   * The projects in the reactor.
   * @parameter expression="${reactorProjects}"
   * @readonly
   */
  private List<MavenProject> reactorProjects;
  
  /**
   * The maven project running this plugin
   *
   * @parameter expression="${project}"
   * @required
   * @readonly
   */
  private MavenProject mavenProject;

  /**
   * JDBC URL
   *
   * @parameter expression="${sonar.jdbc.url}" default-value="jdbc:derby://localhost:1527/sonar" alias="sonar.jdbc.url"
   */
  private String jdbcURL;

  /**
   * JDBC driver class
   *
   * @parameter expression="${sonar.jdbc.driver}" default-value="org.apache.derby.jdbc.ClientDriver" alias="sonar.jdbc.driver"
   */
  private String jdbcDriverClassName;

  /**
   * JDBC user name
   *
   * @parameter expression="${sonar.jdbc.username}" default-value="sonar" alias="sonar.jdbc.username"
   */
  private String jdbcUserName;

  /**
   * JDBC password
   *
   * @parameter expression="${sonar.jdbc.password}" default-value="sonar" alias="sonar.jdbc.password"
   */
  private String jdbcPassword;
  
  /**
   * Project branch
   *
   * @parameter expression="${branch}" default-value="HEAD" alias="branch"
   */
  private String branch;
  
  /**
   * Sonar light mode. Default is false.
   *
   * @parameter expression="${sonar.light}" default-value="false" alias="sonar.light"
   */
  private Boolean light;
  
  private EntityManager manager;
  
  private boolean externalManager = false;
  
  protected void setManager(EntityManager manager) {
    externalManager = true;
    this.manager = manager;
  }
  
  protected EntityManager getEntityManager() throws MojoExecutionException {
    if ( manager == null ) {
      manager = getPersistence().getNewEntityManager();
    }
    return manager;
  }
  
  protected void destroyManager() {
    if (!externalManager ) {
      manager.clear();
      if (manager.isOpen()) {
        manager.close();
        manager = null;
      }
    }
  }
  
  protected void setReactorProjects(List<MavenProject> reactorProjects) {
    this.reactorProjects = reactorProjects;
  }

  protected List<MavenProject> getReactorProjects() {
    return reactorProjects;
  }

  protected String getBranch() {
    return branch;
  }

  protected void setBranch(String branch) {
    this.branch = branch;
  }
  
  protected MavenProject getProject() {
    return this.mavenProject;
  }
  
  protected void setProject(MavenProject mavenProject) {
    this.mavenProject = mavenProject;
  }
  
  protected void setJdbcURL(String jdbcURL) {
    this.jdbcURL = jdbcURL;
  }

  protected void setJdbcDriverClassName(String jdbcDriverClassName) {
    this.jdbcDriverClassName = jdbcDriverClassName;
  }

  protected void setJdbcUserName(String jdbcUserName) {
    this.jdbcUserName = jdbcUserName;
  }

  protected void setJdbcPassword(String jdbcPassword) {
    this.jdbcPassword = jdbcPassword;
  }
  
  protected void setLight(boolean light) {
    this.light = light;
  }
  
  protected Boolean isLight() {
    return this.light != null && this.light;
  }
  
  protected JdbcData getJdbcData() {
    JdbcData jdbcData = new JdbcData(jdbcURL, jdbcDriverClassName, jdbcUserName, jdbcPassword);
    jdbcData.setIsolationLevel(Connection.TRANSACTION_READ_COMMITTED);
    return jdbcData;
  }
  
  protected Integer getSnapshotId(org.apache.maven.project.MavenProject pom) {
    String propValue = (String)pom.getProperties().get(SNAPSHOT_ID_PROP);
    if ( propValue == null ) {
      throw new IllegalStateException("Unable to retreive property '" + SNAPSHOT_ID_PROP + "' from pom");
    }
    return new Integer( propValue );
  }
  
  protected void setSnapshotId(Integer id, org.apache.maven.project.MavenProject pom) {
    pom.getProperties().setProperty(SNAPSHOT_ID_PROP, id.toString());
  }
  
  private Persistence getPersistence() throws MojoExecutionException {
    Persistence persistence;
    try {
      persistence = Persistence.create(getJdbcData());
    } catch (PersistenceException e) {
      throw new MojoExecutionException("DB model exception occured", e);
    }
    return persistence;
  }
}
