/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.Plugin;
import org.apache.maven.model.Repository;
import org.apache.maven.model.RepositoryPolicy;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import ch.hortis.sonar.mvn.reports.ReportHandler;
import ch.hortis.sonar.service.WebInterfaceService;
import ch.hortis.sonar.service.WebInterfaceServiceImpl;

/**
 * @goal prepare
 */
public class PrepareMojo extends AbstractMojo {

  /**
   * The maven project running this plugin
   * @parameter expression="${project}"
   * @required
   * @readonly
   */
  private org.apache.maven.project.MavenProject mavenProject;
  
  /**
   * Sonar host URL
   * @parameter expression="${sonar.host.url}" default-value="http://localhost:9000" alias="sonar.host.url"
   */
  private String sonarHostURL;
  
  /**
   * Sonar server version
   * @parameter expression="${sonar.server.version}" alias="sonar.server.version"
   */
  private String sonarServerVersion;
  
  /**
   * Sonar light mode. Default is false.
   *
   * @parameter expression="${sonar.light}" default-value="false" alias="sonar.light"
   */
  private Boolean light;

  private WebInterfaceService webInterfaceService;
  
  private MavenContext mavenContext;
  
  public PrepareMojo() {	 
  }
  
  public void setWebInterfaceService( WebInterfaceService webInterfaceService ) {
    this.webInterfaceService = webInterfaceService;
  }
  
  public MavenContext getMavenContext() {
      return mavenContext;
  }
  
  public void setMavenProject(org.apache.maven.project.MavenProject mavenProject) {
    this.mavenProject = mavenProject;
  }
  
  protected List<Report> getTargetReports() {
    if (light!=null && light){
     return Report.getLightReports(); 
    }
    return Report.getReports(); 
  }

  public void execute() throws MojoExecutionException {
    if ( webInterfaceService == null ) {
      webInterfaceService = new WebInterfaceServiceImpl( sonarHostURL );
    }         
    mavenContext = new MavenContext(mavenProject, webInterfaceService, this.getClass().getClassLoader());       
    preparePomForSonar();    
    preparePomForRulesExtensions();
    preparePomForReports();
    generateGoals();
  }

  private void generateGoals() throws MojoExecutionException {
    boolean hasJavaSources = !PomUtils.getJavaSourceFiles(mavenProject).isEmpty();
    boolean hasJavaTests = !PomUtils.getJavaTestFiles(mavenProject).isEmpty();

    for (Report report : getTargetReports()) {
      ReportHandler reportHandler = report.getReportHandler();
      if (reportHandler!=null) {
        if ((hasJavaSources || reportHandler.executeEvenIfNoJavaSources())
            && (hasJavaTests || reportHandler.executeEvenIfNoJavaTests()) &&
            reportHandler.execute( mavenContext, getLog() ) ) {
          setReportGoals(reportHandler);
        }
      }
    }
  }

  private void setReportGoals(ReportHandler reportHandler) throws MojoExecutionException {
    for (String command : reportHandler.getCommands()) {
      String goals = mavenProject.getProperties().getProperty("sonar.bootstrap.additional.goals");
      if ( goals == null ) goals = "";
      goals += command + "," + reportHandler.mustFailOnError() + ";";
      mavenProject.getProperties().setProperty("sonar.bootstrap.additional.goals", goals);
    }
  }
  
  private void preparePomForRulesExtensions() throws MojoExecutionException {
    try {
      if (webInterfaceService.hasCheckstyleRulesExtensions()) {
        Report rulesReport = Report.Checkstyle;
        addDependencyToPlugin(rulesReport.getGroupId(), rulesReport.getArtifactId(), rulesReport.getVersion(),
            "org.codehaus.sonar.extensions.runtime.rules", "checkstyle", "1.0-SNAPSHOT", "pom");
      }
      if (webInterfaceService.hasPmdRulesExtensions()) {
        Report rulesReport = Report.PMD;
        addDependencyToPlugin(rulesReport.getGroupId(), rulesReport.getArtifactId(), rulesReport.getVersion(),
            "org.codehaus.sonar.extensions.runtime.rules", "pmd", "1.0-SNAPSHOT", "pom");
      }
    } catch (IOException e) {
      throw new MojoExecutionException("Unable to add rules extensions plugin", e);
    }
  }

  private void addDependencyToPlugin(String pluginGroupId, String pluginArtifactId, String pluginVersion,
                                    String depGroupId, String depArtifactId, String depVersion, String dependencyType) throws MojoExecutionException {
    
    boolean pluginAlreadyDefined = true;
    Plugin plugin = PomUtils.findBuildPlugin(mavenProject, pluginGroupId, pluginArtifactId);
    if ( plugin != null && pluginVersion !=null && !pluginVersion.equals(plugin.getVersion())) {
      plugin = null;
    }
    if (plugin == null) {
      pluginAlreadyDefined = false;
      plugin = new Plugin();
      plugin.setGroupId(pluginGroupId);
      plugin.setArtifactId(pluginArtifactId);
      plugin.setVersion(pluginVersion);
    }

    Dependency dependency = new Dependency();
    dependency.setGroupId(depGroupId);
    dependency.setArtifactId(depArtifactId);
    dependency.setVersion(depVersion);
    dependency.setType(dependencyType);
    plugin.addDependency(dependency);
    if (!pluginAlreadyDefined) {
      mavenProject.getBuild().addPlugin(plugin);
    }
  }

  private void preparePomForReports() throws MojoExecutionException {
    File targetDir = new File( mavenProject.getBuild().getDirectory() + "/sonar" );
    if (!targetDir.exists() && !targetDir.mkdirs()) {
      throw new MojoExecutionException( "Unable to created directory " + targetDir.getPath() );
    }

    for (Report report : getTargetReports()) {
      ReportHandler reportHandler = report.getReportHandler();
      reportHandler.preparePom(mavenProject);
    }
    if (mavenProject.getReporting() != null) {
      mavenProject.getReporting().setPlugins(null);
    }
    //  avoid a bug with maven when artifact versionRange is null
    // should be fixed with maven 2.0.7 (bug with 2.0.6)
    if ( mavenProject.getArtifact() != null && 
         mavenProject.getArtifact().getVersionRange() == null ) {
      getLog().warn( "Project has no version range" );
      MavenProject parent = mavenProject;
      while ( parent.getParent() != null ) {
        parent = parent.getParent();
        if ( parent.getVersion() != null ) {
          mavenProject.getArtifact().setVersionRange( VersionRange.createFromVersion( parent.getVersion() ) );
        }
      }
      if ( mavenProject.getArtifact().getVersionRange() == null ) {
        getLog().warn( "Unable to define POM artifact version, please provide it in the project POM, forcing to version 1.0" );
        mavenProject.getArtifact().setVersionRange( VersionRange.createFromVersion( "1.0" ) );
      } else {
        getLog().info( "Pom version set to " + mavenProject.getArtifact().getVersionRange() );
      }
    }
  }
  
  private void preparePomForSonar() throws MojoExecutionException {
    // adding the repository
    List<Repository> repositories = mavenProject.getRepositories();
    for (Repository repository : repositories) {
      if ( repository.getId().equals("sonar") ) {
        throw new MojoExecutionException("Pom already contains a repository with id 'sonar', please remove it");
      }
    }

    Repository sonarRepository = new Repository();
    sonarRepository.setId("sonar");
    sonarRepository.setName("Sonar server maven repository");
    RepositoryPolicy snapshotPolicy = new RepositoryPolicy();
    snapshotPolicy.setChecksumPolicy("ignore");
    snapshotPolicy.setEnabled(true);
    snapshotPolicy.setUpdatePolicy("always");
    sonarRepository.setSnapshots(snapshotPolicy);
    sonarRepository.setUrl(webInterfaceService.getSonarMavenRepository());
    repositories.add(sonarRepository);

    addDependencyToPlugin("org.codehaus.sonar", "sonar-core-maven-plugin", sonarServerVersion,
                          "org.codehaus.sonar.extensions.runtime", "jdbc-driver", "1.0-SNAPSHOT", "jar");
  }

  protected void setLight(Boolean light) {
    this.light = light;
  }
  
}