/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.mc;

import ch.hortis.sonar.model.Collectable;
import ch.hortis.sonar.model.Metrics;
import ch.hortis.sonar.model.ProjectMeasure;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.w3c.dom.Element;

import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class JDependsCollector extends BaseMeasuresCollector {
  protected XmlReportParser parser;

  public boolean initialize( MavenProject project ) {
    File report = findFileFromBuildDirectory( project, "jdepend-report.xml" );
    if ( report != null ) {
      parser = new XmlReportParser();
      parser.parse( report );
      return true;
    }
    return false;
  }

  public List<Collectable> collect() throws MojoExecutionException {
    ArrayList<Collectable> result = new ArrayList<Collectable>();
    result.addAll( collectProjectMeasures() );
    return result;
  }

  public List<ProjectMeasure> collectProjectMeasures() throws MojoExecutionException {
    List<ProjectMeasure> measures = new ArrayList<ProjectMeasure>();
    double cyclicDependencies = 0, totalClasses = 0, concreteClasses = 0, abstractClasses = 0;
    double ca = 0, ce = 0, i = 0, a = 0, d = 0, v = 0;

    Element packagesRoot = parser.getChildElement( parser.getRoot(), "Packages" );
    if ( packagesRoot != null ) {
      List<Element> packages = parser.getChildElements( packagesRoot, "Package" );
      for (Element packageEl : packages) {

        if ( parser.getChildElement( packageEl, "error" ) != null ) continue;

        Element statsEl = parser.getChildElement( packageEl, "Stats" );
        if ( statsEl == null ) continue;
        try {
          totalClasses += parseNumber( parser.getChildElementValue( statsEl, "TotalClasses" ) );
          concreteClasses += parseNumber( parser.getChildElementValue( statsEl, "ConcreteClasses" ) );
          abstractClasses += parseNumber( parser.getChildElementValue( statsEl, "AbstractClasses" ) );
          ca += parseNumber( parser.getChildElementValue( statsEl, "Ca" ) );
          ce += parseNumber( parser.getChildElementValue( statsEl, "Ce" ) );
          a += parseNumber( parser.getChildElementValue( statsEl, "A" ) );
          i += parseNumber( parser.getChildElementValue( statsEl, "I" ) );
          d += parseNumber( parser.getChildElementValue( statsEl, "D" ) );
          v += parseNumber( parser.getChildElementValue( statsEl, "V" ) );
        } catch (ParseException ex) {
          throw new MojoExecutionException( "ParseException during Jdepends report parsing", ex );
        }
      }
    }

    Element cyclesRoot = parser.getChildElement( parser.getRoot(), "Cycles" );
    if ( cyclesRoot != null ) {
      List<Element> packages = parser.getChildElements( cyclesRoot, "Package" );
      if (!packages.isEmpty()) {
        for (Element packageEl : packages) {
          // has always 2 xml elems for 1 dependency
          cyclicDependencies += parser.getChildElements( packageEl, "Package" ).size() / 2;
        }
      }
    }

    addMeasure( measures, Metrics.JDEPEND_TOTAL_CLASSES, totalClasses );
    addMeasure( measures, Metrics.JDEPEND_CONCRETE_CLASSES, concreteClasses );
    addMeasure( measures, Metrics.JDEPEND_ABSTRACT_CLASSES, abstractClasses );
    addMeasure( measures, Metrics.JDEPEND_AFFERENT_COUPLINGS, ca );
    addMeasure( measures, Metrics.JDEPEND_EFFERENT_COUPLINGS, ce );
    addMeasure( measures, Metrics.JDEPEND_ABSTRACTNESS, a );
    addMeasure( measures, Metrics.JDEPEND_INSTABILITY, i );
    addMeasure( measures, Metrics.JDEPEND_DISTANCE, d );
    addMeasure( measures, Metrics.JDEPEND_VOLATILITY, v );
    addMeasure( measures, Metrics.JDEPEND_PACKAGE_DEPENDENCY_CYCLES, cyclicDependencies );
    return measures;
  }

  private void addMeasure( Collection<ProjectMeasure> measures, Metrics metricEnum, double ce ) {
    ProjectMeasure measure;
    measure = new ProjectMeasure();
    measure.setMetric( loadMetric( metricEnum ) );
    measure.setValue( ce );
    measures.add( measure );
  }



}
